/****************************************************************************
** Meta object code from reading C++ file 'pimdatabackuprestore.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.1)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../pim-data-exporter-26.04.3/core/pimdatabackuprestore.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'pimdatabackuprestore.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.1. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN20PimDataBackupRestoreE_t {};
} // unnamed namespace

template <> constexpr inline auto PimDataBackupRestore::qt_create_metaobjectdata<qt_meta_tag_ZN20PimDataBackupRestoreE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "PimDataBackupRestore",
        "addInfo",
        "",
        "addEndLine",
        "updateActions",
        "state",
        "addError",
        "addTitle",
        "jobFinished",
        "backupDone",
        "restoreDone",
        "jobFailed",
        "needSyncResource",
        "name",
        "identifier"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'addInfo'
        QtMocHelpers::SignalData<void(const QString &)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 2 },
        }}),
        // Signal 'addEndLine'
        QtMocHelpers::SignalData<void()>(3, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'updateActions'
        QtMocHelpers::SignalData<void(bool)>(4, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 5 },
        }}),
        // Signal 'addError'
        QtMocHelpers::SignalData<void(const QString &)>(6, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 2 },
        }}),
        // Signal 'addTitle'
        QtMocHelpers::SignalData<void(const QString &)>(7, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 2 },
        }}),
        // Signal 'jobFinished'
        QtMocHelpers::SignalData<void()>(8, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'backupDone'
        QtMocHelpers::SignalData<void()>(9, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'restoreDone'
        QtMocHelpers::SignalData<void()>(10, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'jobFailed'
        QtMocHelpers::SignalData<void()>(11, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'needSyncResource'
        QtMocHelpers::SignalData<void(const QString &, const QString &)>(12, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 13 }, { QMetaType::QString, 14 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<PimDataBackupRestore, qt_meta_tag_ZN20PimDataBackupRestoreE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject PimDataBackupRestore::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN20PimDataBackupRestoreE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN20PimDataBackupRestoreE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN20PimDataBackupRestoreE_t>.metaTypes,
    nullptr
} };

void PimDataBackupRestore::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<PimDataBackupRestore *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->addInfo((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 1: _t->addEndLine(); break;
        case 2: _t->updateActions((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 3: _t->addError((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 4: _t->addTitle((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 5: _t->jobFinished(); break;
        case 6: _t->backupDone(); break;
        case 7: _t->restoreDone(); break;
        case 8: _t->jobFailed(); break;
        case 9: _t->needSyncResource((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2]))); break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (PimDataBackupRestore::*)(const QString & )>(_a, &PimDataBackupRestore::addInfo, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (PimDataBackupRestore::*)()>(_a, &PimDataBackupRestore::addEndLine, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (PimDataBackupRestore::*)(bool )>(_a, &PimDataBackupRestore::updateActions, 2))
            return;
        if (QtMocHelpers::indexOfMethod<void (PimDataBackupRestore::*)(const QString & )>(_a, &PimDataBackupRestore::addError, 3))
            return;
        if (QtMocHelpers::indexOfMethod<void (PimDataBackupRestore::*)(const QString & )>(_a, &PimDataBackupRestore::addTitle, 4))
            return;
        if (QtMocHelpers::indexOfMethod<void (PimDataBackupRestore::*)()>(_a, &PimDataBackupRestore::jobFinished, 5))
            return;
        if (QtMocHelpers::indexOfMethod<void (PimDataBackupRestore::*)()>(_a, &PimDataBackupRestore::backupDone, 6))
            return;
        if (QtMocHelpers::indexOfMethod<void (PimDataBackupRestore::*)()>(_a, &PimDataBackupRestore::restoreDone, 7))
            return;
        if (QtMocHelpers::indexOfMethod<void (PimDataBackupRestore::*)()>(_a, &PimDataBackupRestore::jobFailed, 8))
            return;
        if (QtMocHelpers::indexOfMethod<void (PimDataBackupRestore::*)(const QString & , const QString & )>(_a, &PimDataBackupRestore::needSyncResource, 9))
            return;
    }
}

const QMetaObject *PimDataBackupRestore::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *PimDataBackupRestore::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN20PimDataBackupRestoreE_t>.strings))
        return static_cast<void*>(this);
    return QObject::qt_metacast(_clname);
}

int PimDataBackupRestore::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 10)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 10;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 10)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 10;
    }
    return _id;
}

// SIGNAL 0
void PimDataBackupRestore::addInfo(const QString & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 0, nullptr, _t1);
}

// SIGNAL 1
void PimDataBackupRestore::addEndLine()
{
    QMetaObject::activate(this, &staticMetaObject, 1, nullptr);
}

// SIGNAL 2
void PimDataBackupRestore::updateActions(bool _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 2, nullptr, _t1);
}

// SIGNAL 3
void PimDataBackupRestore::addError(const QString & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 3, nullptr, _t1);
}

// SIGNAL 4
void PimDataBackupRestore::addTitle(const QString & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 4, nullptr, _t1);
}

// SIGNAL 5
void PimDataBackupRestore::jobFinished()
{
    QMetaObject::activate(this, &staticMetaObject, 5, nullptr);
}

// SIGNAL 6
void PimDataBackupRestore::backupDone()
{
    QMetaObject::activate(this, &staticMetaObject, 6, nullptr);
}

// SIGNAL 7
void PimDataBackupRestore::restoreDone()
{
    QMetaObject::activate(this, &staticMetaObject, 7, nullptr);
}

// SIGNAL 8
void PimDataBackupRestore::jobFailed()
{
    QMetaObject::activate(this, &staticMetaObject, 8, nullptr);
}

// SIGNAL 9
void PimDataBackupRestore::needSyncResource(const QString & _t1, const QString & _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 9, nullptr, _t1, _t2);
}
QT_WARNING_POP
