// This file is generated by kconfig_compiler_kf6 from pimdataexporterglobalconfig.kcfg.
// All changes you do to this file will be lost.
#ifndef PIMDATAEXPORTGLOBALCONFIG_H
#define PIMDATAEXPORTGLOBALCONFIG_H

#include <kconfigskeleton.h>
#include <QCoreApplication>
#include <QDebug>

class PimDataExportGlobalConfig : public KConfigSkeleton
{
  public:

    static PimDataExportGlobalConfig *self();
    ~PimDataExportGlobalConfig() override;

    /**
      Set AlwaysOverrideFile
    */
    static
    void setAlwaysOverrideFile( bool v )
    {
      if (!self()->isAlwaysOverrideFileImmutable())
        self()->mAlwaysOverrideFile = v;
    }

    /**
      Get AlwaysOverrideFile
    */
    static
    bool alwaysOverrideFile()
    {
      return self()->mAlwaysOverrideFile;
    }

    /**
      Is AlwaysOverrideFile Immutable
    */
    static
    bool isAlwaysOverrideFileImmutable()
    {
      return self()->isImmutable( QStringLiteral( "AlwaysOverrideFile" ) );
    }

    /**
      Get Item object corresponding to AlwaysOverrideFile()
    */
    ItemBool *alwaysOverrideFileItem()
    {
      return mAlwaysOverrideFileItem;
    }

    /**
      Set AlwaysMergeConfigFile
    */
    static
    void setAlwaysMergeConfigFile( bool v )
    {
      if (!self()->isAlwaysMergeConfigFileImmutable())
        self()->mAlwaysMergeConfigFile = v;
    }

    /**
      Get AlwaysMergeConfigFile
    */
    static
    bool alwaysMergeConfigFile()
    {
      return self()->mAlwaysMergeConfigFile;
    }

    /**
      Is AlwaysMergeConfigFile Immutable
    */
    static
    bool isAlwaysMergeConfigFileImmutable()
    {
      return self()->isImmutable( QStringLiteral( "AlwaysMergeConfigFile" ) );
    }

    /**
      Get Item object corresponding to AlwaysMergeConfigFile()
    */
    ItemBool *alwaysMergeConfigFileItem()
    {
      return mAlwaysMergeConfigFileItem;
    }

    /**
      Set AlwaysOverrideDirectory
    */
    static
    void setAlwaysOverrideDirectory( bool v )
    {
      if (!self()->isAlwaysOverrideDirectoryImmutable())
        self()->mAlwaysOverrideDirectory = v;
    }

    /**
      Get AlwaysOverrideDirectory
    */
    static
    bool alwaysOverrideDirectory()
    {
      return self()->mAlwaysOverrideDirectory;
    }

    /**
      Is AlwaysOverrideDirectory Immutable
    */
    static
    bool isAlwaysOverrideDirectoryImmutable()
    {
      return self()->isImmutable( QStringLiteral( "AlwaysOverrideDirectory" ) );
    }

    /**
      Get Item object corresponding to AlwaysOverrideDirectory()
    */
    ItemBool *alwaysOverrideDirectoryItem()
    {
      return mAlwaysOverrideDirectoryItem;
    }

  protected:
    PimDataExportGlobalConfig();
    friend class PimDataExportGlobalConfigHelper;


    // General
    bool mAlwaysOverrideFile;
    bool mAlwaysMergeConfigFile;
    bool mAlwaysOverrideDirectory;

  private:
    ItemBool *mAlwaysOverrideFileItem;
    ItemBool *mAlwaysMergeConfigFileItem;
    ItemBool *mAlwaysOverrideDirectoryItem;
};

#endif

