// This file is generated by kconfig_compiler_kf6 from pimdataexporterglobalconfig.kcfg.
// All changes you do to this file will be lost.

#include "pimdataexporterglobalconfig.h"

#include <qglobal.h>
#include <QFile>

class PimDataExportGlobalConfigHelper
{
  public:
    PimDataExportGlobalConfigHelper() noexcept : q(nullptr) {}
    ~PimDataExportGlobalConfigHelper() noexcept { delete q; q = nullptr; }
    PimDataExportGlobalConfigHelper(const PimDataExportGlobalConfigHelper&) = delete;
    PimDataExportGlobalConfigHelper& operator=(const PimDataExportGlobalConfigHelper&) = delete;
    PimDataExportGlobalConfig *q;
};
Q_GLOBAL_STATIC(PimDataExportGlobalConfigHelper, s_globalPimDataExportGlobalConfig)
PimDataExportGlobalConfig *PimDataExportGlobalConfig::self()
{
  if (!s_globalPimDataExportGlobalConfig()->q) {
    new PimDataExportGlobalConfig;
    s_globalPimDataExportGlobalConfig()->q->read();
  }

  return s_globalPimDataExportGlobalConfig()->q;
}

PimDataExportGlobalConfig::PimDataExportGlobalConfig( )
  : KConfigSkeleton()
{
  Q_ASSERT(!s_globalPimDataExportGlobalConfig()->q);
  s_globalPimDataExportGlobalConfig()->q = this;
  setCurrentGroup( QStringLiteral( "General" ) );

  mAlwaysOverrideFileItem = new KConfigSkeleton::ItemBool( currentGroup(), QStringLiteral( "AlwaysOverrideFile" ), mAlwaysOverrideFile, false );
  addItem( mAlwaysOverrideFileItem, QStringLiteral( "AlwaysOverrideFile" ) );
  mAlwaysMergeConfigFileItem = new KConfigSkeleton::ItemBool( currentGroup(), QStringLiteral( "AlwaysMergeConfigFile" ), mAlwaysMergeConfigFile, false );
  addItem( mAlwaysMergeConfigFileItem, QStringLiteral( "AlwaysMergeConfigFile" ) );
  mAlwaysOverrideDirectoryItem = new KConfigSkeleton::ItemBool( currentGroup(), QStringLiteral( "AlwaysOverrideDirectory" ), mAlwaysOverrideDirectory, false );
  addItem( mAlwaysOverrideDirectoryItem, QStringLiteral( "AlwaysOverrideDirectory" ) );
}

PimDataExportGlobalConfig::~PimDataExportGlobalConfig()
{
  if (s_globalPimDataExportGlobalConfig.exists() && !s_globalPimDataExportGlobalConfig.isDestroyed()) {
    s_globalPimDataExportGlobalConfig()->q = nullptr;
  }
}

